# Import all metrics
from django_prometheus.db.metrics import (
    Counter,
    connection_errors_total,
    connections_total,
    errors_total,
    execute_many_total,
    execute_total,
    query_duration_seconds,
)

__all__ = [
    "Counter",
    "connection_errors_total",
    "connections_total",
    "errors_total",
    "execute_many_total",
    "execute_total",
    "query_duration_seconds",
]



import psycopg2.extensions
from django.contrib.gis.db.backends.postgis import base

from django_prometheus.db.common import DatabaseWrapperMixin, ExportingCursorWrapper


class DatabaseWrapper(DatabaseWrapperMixin, base.DatabaseWrapper):
    def get_connection_params(self):
        conn_params = super().get_connection_params()
        conn_params["cursor_factory"] = ExportingCursorWrapper(
            psycopg2.extensions.cursor, "postgis", self.vendor
        )
        return conn_params

    def create_cursor(self, name=None):
        # cursor_factory is a kwarg to connect() so restore create_cursor()'s
        # default behavior
        return base.DatabaseWrapper.create_cursor(self, name=name)



from django.test import SimpleTestCase, override_settings
from testapp.views import ObjectionException

from django_prometheus.testutils import PrometheusTestCaseMixin


def M(metric_name):
    """Makes a full metric name from a short metric name.

    This is just intended to help keep the lines shorter in test
    cases.
    """
    return "django_http_%s" % metric_name


def T(metric_name):
    """Makes a full metric name from a short metric name like M(metric_name)

    This method adds a '_total' postfix for metrics."""
    return "%s_total" % M(metric_name)


@override_settings(
    PROMETHEUS_LATENCY_BUCKETS=(0.05, 1.0, 2.0, 4.0, 5.0, 10.0, float("inf"))
)
class TestMiddlewareMetrics(PrometheusTestCaseMixin, SimpleTestCase):
    """Test django_prometheus.middleware.

    Note that counters related to exceptions can't be tested as
    Django's test Client only simulates requests and the exception
    handling flow is very different in that simulation.
    """

    def test_request_counters(self):
        registry = self.saveRegistry()
        self.client.get("/")
        self.client.get("/")
        self.client.get("/help")
        self.client.post("/", {"test": "data"})

        self.assertMetricDiff(registry, 4, M("requests_before_middlewares_total"))
        self.assertMetricDiff(registry, 4, M("responses_before_middlewares_total"))
        self.assertMetricDiff(registry, 3, T("requests_total_by_method"), method="GET")
        self.assertMetricDiff(registry, 1, T("requests_total_by_method"), method="POST")
        self.assertMetricDiff(
            registry, 4, T("requests_total_by_transport"), transport="http"
        )
        self.assertMetricDiff(
            registry,
            2,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.index",
            transport="http",
            method="GET",
        )
        self.assertMetricDiff(
            registry,
            1,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.help",
            transport="http",
            method="GET",
        )
        self.assertMetricDiff(
            registry,
            1,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.index",
            transport="http",
            method="POST",
        )
        # We have 3 requests with no post body, and one with a few
        # bytes, but buckets are cumulative so that is 4 requests with
        # <=128 bytes bodies.
        self.assertMetricDiff(
            registry, 3, M("requests_body_total_bytes_bucket"), le="0.0"
        )
        self.assertMetricDiff(
            registry, 4, M("requests_body_total_bytes_bucket"), le="128.0"
        )
        self.assertMetricEquals(
            None, M("responses_total_by_templatename"), templatename="help.html"
        )
        self.assertMetricDiff(
            registry, 3, T("responses_total_by_templatename"), templatename="index.html"
        )
        self.assertMetricDiff(registry, 4, T("responses_total_by_status"), status="200")
        self.assertMetricDiff(
            registry, 0, M("responses_body_total_bytes_bucket"), le="0.0"
        )
        self.assertMetricDiff(
            registry, 3, M("responses_body_total_bytes_bucket"), le="128.0"
        )
        self.assertMetricDiff(
            registry, 4, M("responses_body_total_bytes_bucket"), le="8192.0"
        )
        self.assertMetricDiff(
            registry, 4, T("responses_total_by_charset"), charset="utf-8"
        )
        self.assertMetricDiff(registry, 0, M("responses_streaming_total"))

    def test_latency_histograms(self):
        # Caution: this test is timing-based. This is not ideal. It
        # runs slowly (each request to /slow takes at least .1 seconds
        # to complete), to eliminate flakiness we adjust the buckets used
        # in the test suite.

        registry = self.saveRegistry()

        # This always takes more than .1 second, so checking the lower
        # buckets is fine.
        self.client.get("/slow")
        self.assertMetricDiff(
            registry,
            0,
            M("requests_latency_seconds_by_view_method_bucket"),
            le="0.05",
            view="slow",
            method="GET",
        )
        self.assertMetricDiff(
            registry,
            1,
            M("requests_latency_seconds_by_view_method_bucket"),
            le="5.0",
            view="slow",
            method="GET",
        )

    def test_exception_latency_histograms(self):
        registry = self.saveRegistry()

        try:
            self.client.get("/objection")
        except ObjectionException:
            pass
        self.assertMetricDiff(
            registry,
            2,
            M("requests_latency_seconds_by_view_method_bucket"),
            le="2.0",
            view="testapp.views.objection",
            method="GET",
        )

    def test_streaming_responses(self):
        registry = self.saveRegistry()
        self.client.get("/")
        self.client.get("/file")
        self.assertMetricDiff(registry, 1, M("responses_streaming_total"))
        self.assertMetricDiff(
            registry, 1, M("responses_body_total_bytes_bucket"), le="+Inf"
        )



from django_memcached_consul import memcached

from django_prometheus.cache.metrics import (
    django_cache_get_total,
    django_cache_hits_total,
    django_cache_misses_total,
)


class MemcachedCache(memcached.MemcachedCache):
    """Inherit django_memcached_consul to add metrics about hit/miss ratio"""

    def get(self, key, default=None, version=None):
        django_cache_get_total.labels(backend="django_memcached_consul").inc()
        cached = super().get(key, default=None, version=version)
        if cached is not None:
            django_cache_hits_total.labels(backend="django_memcached_consul").inc()
        else:
            django_cache_misses_total.labels(backend="django_memcached_consul").inc()
        return cached or default



from unittest import skipUnless

from django.db import connections
from django.test import TestCase

from django_prometheus.testutils import PrometheusTestCaseMixin


class BaseDbMetricTest(PrometheusTestCaseMixin, TestCase):
    # https://docs.djangoproject.com/en/2.2/topics/testing/tools/#django.test.SimpleTestCase.databases
    databases = "__all__"


@skipUnless(
    connections["test_db_1"].vendor == "sqlite", "Skipped unless test_db_1 uses sqlite"
)
class TestDbMetrics(BaseDbMetricTest):
    """Test django_prometheus.db metrics.

    Note regarding the values of metrics: many tests interact with the
    database, and the test runner itself does. As such, tests that
    require that a metric has a specific value are at best very
    fragile. Consider asserting that the value exceeds a certain
    threshold, or check by how much it increased during the test.
    """

    def testConfigHasExpectedDatabases(self):
        """Not a real unit test: ensures that testapp.settings contains the
        databases this test expects."""
        assert "default" in connections.databases.keys()
        assert "test_db_1" in connections.databases.keys()
        assert "test_db_2" in connections.databases.keys()

    def testCounters(self):
        cursor_db1 = connections["test_db_1"].cursor()
        cursor_db2 = connections["test_db_2"].cursor()
        cursor_db1.execute("SELECT 1")
        for _ in range(200):
            cursor_db2.execute("SELECT 2")
        cursor_db1.execute("SELECT 3")
        try:
            cursor_db1.execute("this is clearly not valid SQL")
        except Exception:
            pass

        self.assertMetricEquals(
            1,
            "django_db_errors_total",
            alias="test_db_1",
            vendor="sqlite",
            type="OperationalError",
        )
        assert (
            self.getMetric(
                "django_db_execute_total", alias="test_db_1", vendor="sqlite"
            )
            > 0
        )
        assert (
            self.getMetric(
                "django_db_execute_total", alias="test_db_2", vendor="sqlite"
            )
            >= 200
        )

    def testHistograms(self):
        cursor_db1 = connections["test_db_1"].cursor()
        cursor_db2 = connections["test_db_2"].cursor()
        cursor_db1.execute("SELECT 1")
        for _ in range(200):
            cursor_db2.execute("SELECT 2")
        assert (
            self.getMetric(
                "django_db_query_duration_seconds_count",
                alias="test_db_1",
                vendor="sqlite",
            )
            > 0
        )
        assert (
            self.getMetric(
                "django_db_query_duration_seconds_count",
                alias="test_db_2",
                vendor="sqlite",
            )
            >= 200
        )

    def testExecuteMany(self):
        registry = self.saveRegistry()
        cursor_db1 = connections["test_db_1"].cursor()
        cursor_db1.executemany(
            "INSERT INTO testapp_lawn(location) VALUES (?)",
            [("Paris",), ("New York",), ("Berlin",), ("San Francisco",)],
        )
        self.assertMetricDiff(
            registry,
            4,
            "django_db_execute_many_total",
            alias="test_db_1",
            vendor="sqlite",
        )


@skipUnless(
    "postgresql" in connections, "Skipped unless postgresql database is enabled"
)
class TestPostgresDbMetrics(BaseDbMetricTest):
    """Test django_prometheus.db metrics for postgres backend.

    Note regarding the values of metrics: many tests interact with the
    database, and the test runner itself does. As such, tests that
    require that a metric has a specific value are at best very
    fragile. Consider asserting that the value exceeds a certain
    threshold, or check by how much it increased during the test.
    """

    def testCounters(self):
        registry = self.saveRegistry()
        cursor = connections["postgresql"].cursor()

        for _ in range(20):
            cursor.execute("SELECT 1")

        self.assertMetricCompare(
            registry,
            lambda a, b: a + 20 <= b < a + 25,
            "django_db_execute_total",
            alias="postgresql",
            vendor="postgresql",
        )


@skipUnless("mysql" in connections, "Skipped unless mysql database is enabled")
class TestMysDbMetrics(BaseDbMetricTest):
    """Test django_prometheus.db metrics for mys backend.

    Note regarding the values of metrics: many tests interact with the
    database, and the test runner itself does. As such, tests that
    require that a metric has a specific value are at best very
    fragile. Consider asserting that the value exceeds a certain
    threshold, or check by how much it increased during the test.
    """

    def testCounters(self):
        registry = self.saveRegistry()
        cursor = connections["mysql"].cursor()

        for _ in range(20):
            cursor.execute("SELECT 1")

        self.assertMetricCompare(
            registry,
            lambda a, b: a + 20 <= b < a + 25,
            "django_db_execute_total",
            alias="mysql",
            vendor="mysql",
        )


@skipUnless("postgis" in connections, "Skipped unless postgis database is enabled")
class TestPostgisDbMetrics(BaseDbMetricTest):
    """Test django_prometheus.db metrics for postgis backend.

    Note regarding the values of metrics: many tests interact with the
    database, and the test runner itself does. As such, tests that
    require that a metric has a specific value are at best very
    fragile. Consider asserting that the value exceeds a certain
    threshold, or check by how much it increased during the test.
    """

    def testCounters(self):
        r = self.saveRegistry()
        cursor = connections["postgis"].cursor()

        for _ in range(20):
            cursor.execute("SELECT 1")

        self.assertMetricCompare(
            r,
            lambda a, b: a + 20 <= b < a + 25,
            "django_db_execute_total",
            alias="postgis",
            vendor="postgresql",
        )



from prometheus_client import Counter

from django_prometheus.conf import NAMESPACE

django_cache_get_total = Counter(
    "django_cache_get_total",
    "Total get requests on cache",
    ["backend"],
    namespace=NAMESPACE,
)
django_cache_hits_total = Counter(
    "django_cache_get_hits_total",
    "Total hits on cache",
    ["backend"],
    namespace=NAMESPACE,
)
django_cache_misses_total = Counter(
    "django_cache_get_misses_total",
    "Total misses on cache",
    ["backend"],
    namespace=NAMESPACE,
)
django_cache_get_fail_total = Counter(
    "django_cache_get_fail_total",
    "Total get request failures by cache",
    ["backend"],
    namespace=NAMESPACE,
)



"""Django-Prometheus

https://github.com/korfuri/django-prometheus
"""

# Import all files that define metrics. This has the effect that
# `import django_prometheus` will always instantiate all metric
# objects right away.
import django

from django_prometheus import middleware, models

__all__ = ["middleware", "models", "pip_prometheus"]

__version__ = "2.2.0"

# Import pip_prometheus to export the pip metrics automatically.
try:
    import pip_prometheus
except ImportError:
    # If people don't have pip, don't export anything.
    pass

if django.VERSION < (3, 1):
    default_app_config = "django_prometheus.apps.DjangoPrometheusConfig"



from unittest.mock import MagicMock

from django.test import SimpleTestCase

from django_prometheus.migrations import ExportMigrationsForDatabase
from django_prometheus.testutils import PrometheusTestCaseMixin


def M(metric_name):
    """Make a full metric name from a short metric name.

    This is just intended to help keep the lines shorter in test
    cases.
    """
    return "django_migrations_%s" % metric_name


class TestMigrations(PrometheusTestCaseMixin, SimpleTestCase):
    """Test migration counters."""

    def test_counters(self):
        executor = MagicMock()
        executor.migration_plan = MagicMock()
        executor.migration_plan.return_value = set()
        executor.loader.applied_migrations = {"a", "b", "c"}
        ExportMigrationsForDatabase("fakedb1", executor)
        self.assertEqual(executor.migration_plan.call_count, 1)
        executor.migration_plan = MagicMock()
        executor.migration_plan.return_value = {"a"}
        executor.loader.applied_migrations = {"b", "c"}
        ExportMigrationsForDatabase("fakedb2", executor)

        self.assertMetricEquals(3, M("applied_total"), connection="fakedb1")
        self.assertMetricEquals(0, M("unapplied_total"), connection="fakedb1")
        self.assertMetricEquals(2, M("applied_total"), connection="fakedb2")
        self.assertMetricEquals(1, M("unapplied_total"), connection="fakedb2")



#!/usr/bin/env python
import socket
from unittest.mock import ANY, MagicMock, call, patch

from django_prometheus.exports import SetupPrometheusEndpointOnPortRange


@patch("django_prometheus.exports.HTTPServer")
def test_port_range_available(httpserver_mock):
    """Test port range setup with an available port."""
    httpserver_mock.side_effect = [socket.error, MagicMock()]
    port_range = [8000, 8001]
    port_chosen = SetupPrometheusEndpointOnPortRange(port_range)
    assert port_chosen in port_range

    expected_calls = [call(("", 8000), ANY), call(("", 8001), ANY)]
    assert httpserver_mock.mock_calls == expected_calls


@patch("django_prometheus.exports.HTTPServer")
def test_port_range_unavailable(httpserver_mock):
    """Test port range setup with no available ports."""
    httpserver_mock.side_effect = [socket.error, socket.error]
    port_range = [8000, 8001]
    port_chosen = SetupPrometheusEndpointOnPortRange(port_range)

    expected_calls = [call(("", 8000), ANY), call(("", 8001), ANY)]
    assert httpserver_mock.mock_calls == expected_calls
    assert port_chosen is None



from prometheus_client import Counter

from django_prometheus.conf import NAMESPACE

model_inserts = Counter(
    "django_model_inserts_total",
    "Number of insert operations by model.",
    ["model"],
    namespace=NAMESPACE,
)

model_updates = Counter(
    "django_model_updates_total",
    "Number of update operations by model.",
    ["model"],
    namespace=NAMESPACE,
)

model_deletes = Counter(
    "django_model_deletes_total",
    "Number of delete operations by model.",
    ["model"],
    namespace=NAMESPACE,
)


def ExportModelOperationsMixin(model_name):
    """Returns a mixin for models to export counters for lifecycle operations.

    Usage:
      class User(ExportModelOperationsMixin('user'), Model):
          ...
    """
    # Force create the labels for this model in the counters. This
    # is not necessary but it avoids gaps in the aggregated data.
    model_inserts.labels(model_name)
    model_updates.labels(model_name)
    model_deletes.labels(model_name)

    class Mixin:
        def _do_insert(self, *args, **kwargs):
            model_inserts.labels(model_name).inc()
            return super()._do_insert(*args, **kwargs)

        def _do_update(self, *args, **kwargs):
            model_updates.labels(model_name).inc()
            return super()._do_update(*args, **kwargs)

        def delete(self, *args, **kwargs):
            model_deletes.labels(model_name).inc()
            return super().delete(*args, **kwargs)

    Mixin.__qualname__ = f"ExportModelOperationsMixin('{model_name}')"
    return Mixin



import os
import time

from django.db import connections
from django.http import FileResponse
from django.shortcuts import render
from django.template.response import TemplateResponse
from testapp.models import Lawn


def index(request):
    return TemplateResponse(request, "index.html", {})


def help(request):
    # render does not instantiate a TemplateResponse, so it does not
    # increment the "by_templatename" counters.
    return render(request, "help.html", {})


def slow(request):
    """This view takes .1s to load, on purpose."""
    time.sleep(0.1)
    return TemplateResponse(request, "slow.html", {})


def newlawn(request, location):
    """This view creates a new Lawn instance in the database."""
    lawn = Lawn()
    lawn.location = location
    lawn.save()
    return TemplateResponse(request, "lawn.html", {"lawn": lawn})


class ObjectionException(Exception):
    pass


def objection(request):
    raise ObjectionException("Objection!")


def sql(request):
    databases = connections.databases.keys()
    query = request.GET.get("query")
    db = request.GET.get("database")
    if query and db:
        cursor = connections[db].cursor()
        cursor.execute(query, [])
        results = cursor.fetchall()
        return TemplateResponse(
            request,
            "sql.html",
            {"query": query, "rows": results, "databases": databases},
        )
    else:
        return TemplateResponse(
            request, "sql.html", {"query": None, "rows": None, "databases": databases}
        )


def file(request):
    return FileResponse(open(os.devnull, "rb"))



from django.db.models import CharField, Model, PositiveIntegerField

from django_prometheus.models import ExportModelOperationsMixin


class Dog(ExportModelOperationsMixin("dog"), Model):
    name = CharField(max_length=100, unique=True)
    breed = CharField(max_length=100, blank=True, null=True)
    age = PositiveIntegerField(blank=True, null=True)


class Lawn(ExportModelOperationsMixin("lawn"), Model):
    location = CharField(max_length=100)



import os
import tempfile

from testapp.helpers import get_middleware

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ")0-t%mc5y1^fn8e7i**^^v166@5iu(&-2%9#kxud0&4ap#k!_k"
DEBUG = True
ALLOWED_HOSTS = []


# Application definition
INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django_prometheus",
    "testapp",
)


MIDDLEWARE = get_middleware(
    "django_prometheus.middleware.PrometheusBeforeMiddleware",
    "django_prometheus.middleware.PrometheusAfterMiddleware",
)

ROOT_URLCONF = "testapp.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "testapp.wsgi.application"


DATABASES = {
    "default": {
        "ENGINE": "django_prometheus.db.backends.sqlite3",
        "NAME": "db.sqlite3",
    },
    # Comment this to not test django_prometheus.db.backends.postgres.
    "postgresql": {
        "ENGINE": "django_prometheus.db.backends.postgresql",
        "NAME": "postgres",
        "USER": "postgres",
        "PASSWORD": "",
        "HOST": "localhost",
        "PORT": "5432",
    },
    # Comment this to not test django_prometheus.db.backends.postgis.
    "postgis": {
        "ENGINE": "django_prometheus.db.backends.postgis",
        "NAME": "postgis",
        "USER": "postgres",
        "PASSWORD": "",
        "HOST": "localhost",
        "PORT": "5432",
    },
    # Comment this to not test django_prometheus.db.backends.mysql.
    "mysql": {
        "ENGINE": "django_prometheus.db.backends.mysql",
        "NAME": "django_prometheus_1",
        "USER": "root",
        "PASSWORD": "",
        "HOST": "127.0.0.1",
        "PORT": "3306",
    },
    # The following databases are used by test_db.py only
    "test_db_1": {
        "ENGINE": "django_prometheus.db.backends.sqlite3",
        "NAME": "test_db_1.sqlite3",
    },
    "test_db_2": {
        "ENGINE": "django_prometheus.db.backends.sqlite3",
        "NAME": "test_db_2.sqlite3",
    },
}

# Caches
_tmp_cache_dir = tempfile.mkdtemp()

CACHES = {
    "default": {
        "BACKEND": "django_prometheus.cache.backends.memcached.MemcachedCache",
        "LOCATION": "localhost:11211",
    },
    "memcached": {
        "BACKEND": "django_prometheus.cache.backends.memcached.MemcachedCache",
        "LOCATION": "localhost:11211",
    },
    "filebased": {
        "BACKEND": "django_prometheus.cache.backends.filebased.FileBasedCache",
        "LOCATION": os.path.join(_tmp_cache_dir, "django_cache"),
    },
    "locmem": {
        "BACKEND": "django_prometheus.cache.backends.locmem.LocMemCache",
        "LOCATION": os.path.join(_tmp_cache_dir, "locmem_cache"),
    },
    "redis": {
        "BACKEND": "django_prometheus.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/1",
    },
    # Fake redis config emulated stopped service
    "stopped_redis": {
        "BACKEND": "django_prometheus.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6666/1",
    },
    "stopped_redis_ignore_exception": {
        "BACKEND": "django_prometheus.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6666/1",
        "OPTIONS": {"IGNORE_EXCEPTIONS": True},
    },
}

# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = False


# Static files (CSS, JavaScript, Images)
STATIC_URL = "/static/"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {"console": {"class": "logging.StreamHandler"}},
    "root": {"handlers": ["console"], "level": "INFO"},
    "loggers": {"django": {"handlers": ["console"], "level": "INFO"}},
}



from django.contrib import admin
from django.urls import include, path, re_path
from testapp import views

urlpatterns = [
    re_path(r"^$", views.index),
    re_path(r"^help$", views.help),
    re_path(r"^slow$", views.slow, name="slow"),
    re_path(r"^objection$", views.objection),
    re_path(r"^sql$", views.sql),
    re_path(r"^newlawn/(?P<location>[a-zA-Z0-9 ]+)$", views.newlawn),
    re_path(r"^file$", views.file),
    path("", include("django_prometheus.urls")),
    re_path(r"^admin/", admin.site.urls),
]



from django.test import TestCase
from testapp.models import Dog, Lawn

from django_prometheus.testutils import PrometheusTestCaseMixin


def M(metric_name):
    """Make a full metric name from a short metric name.

    This is just intended to help keep the lines shorter in test
    cases.
    """
    return "django_model_%s" % metric_name


class TestModelMetrics(PrometheusTestCaseMixin, TestCase):
    """Test django_prometheus.models."""

    def test_counters(self):
        registry = self.saveRegistry()
        cool = Dog()
        cool.name = "Cool"
        cool.save()
        self.assertMetricDiff(registry, 1, M("inserts_total"), model="dog")

        elysees = Lawn()
        elysees.location = "Champs Elysees, Paris"
        elysees.save()
        self.assertMetricDiff(registry, 1, M("inserts_total"), model="lawn")
        self.assertMetricDiff(registry, 1, M("inserts_total"), model="dog")

        galli = Dog()
        galli.name = "Galli"
        galli.save()
        self.assertMetricDiff(registry, 2, M("inserts_total"), model="dog")

        cool.breed = "Wolfhound"
        self.assertMetricDiff(registry, 2, M("inserts_total"), model="dog")

        cool.save()
        self.assertMetricDiff(registry, 2, M("inserts_total"), model="dog")
        self.assertMetricDiff(registry, 1, M("updates_total"), model="dog")

        cool.age = 9
        cool.save()
        self.assertMetricDiff(registry, 2, M("updates_total"), model="dog")

        cool.delete()  # :(
        self.assertMetricDiff(registry, 2, M("inserts_total"), model="dog")
        self.assertMetricDiff(registry, 2, M("updates_total"), model="dog")
        self.assertMetricDiff(registry, 1, M("deletes_total"), model="dog")



from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from prometheus_client import Counter, Histogram

from django_prometheus.conf import NAMESPACE
from django_prometheus.utils import PowersOf, Time, TimeSince

DEFAULT_LATENCY_BUCKETS = (
    0.01,
    0.025,
    0.05,
    0.075,
    0.1,
    0.25,
    0.5,
    0.75,
    1.0,
    2.5,
    5.0,
    7.5,
    10.0,
    25.0,
    50.0,
    75.0,
    float("inf"),
)


class Metrics:
    _instance = None

    @classmethod
    def get_instance(cls):
        if not cls._instance:
            cls._instance = cls()
        return cls._instance

    def register_metric(self, metric_cls, name, documentation, labelnames=(), **kwargs):
        return metric_cls(name, documentation, labelnames=labelnames, **kwargs)

    def __init__(self, *args, **kwargs):
        self.register()

    def register(self):
        self.requests_total = self.register_metric(
            Counter,
            "django_http_requests_before_middlewares_total",
            "Total count of requests before middlewares run.",
            namespace=NAMESPACE,
        )
        self.responses_total = self.register_metric(
            Counter,
            "django_http_responses_before_middlewares_total",
            "Total count of responses before middlewares run.",
            namespace=NAMESPACE,
        )
        self.requests_latency_before = self.register_metric(
            Histogram,
            "django_http_requests_latency_including_middlewares_seconds",
            (
                "Histogram of requests processing time (including middleware "
                "processing time)."
            ),
            namespace=NAMESPACE,
        )
        self.requests_unknown_latency_before = self.register_metric(
            Counter,
            "django_http_requests_unknown_latency_including_middlewares_total",
            (
                "Count of requests for which the latency was unknown (when computing "
                "django_http_requests_latency_including_middlewares_seconds)."
            ),
            namespace=NAMESPACE,
        )
        self.requests_latency_by_view_method = self.register_metric(
            Histogram,
            "django_http_requests_latency_seconds_by_view_method",
            "Histogram of request processing time labelled by view.",
            ["view", "method"],
            buckets=getattr(
                settings, "PROMETHEUS_LATENCY_BUCKETS", DEFAULT_LATENCY_BUCKETS
            ),
            namespace=NAMESPACE,
        )
        self.requests_unknown_latency = self.register_metric(
            Counter,
            "django_http_requests_unknown_latency_total",
            "Count of requests for which the latency was unknown.",
            namespace=NAMESPACE,
        )
        # Set in process_request
        self.requests_ajax = self.register_metric(
            Counter,
            "django_http_ajax_requests_total",
            "Count of AJAX requests.",
            namespace=NAMESPACE,
        )
        self.requests_by_method = self.register_metric(
            Counter,
            "django_http_requests_total_by_method",
            "Count of requests by method.",
            ["method"],
            namespace=NAMESPACE,
        )
        self.requests_by_transport = self.register_metric(
            Counter,
            "django_http_requests_total_by_transport",
            "Count of requests by transport.",
            ["transport"],
            namespace=NAMESPACE,
        )
        # Set in process_view
        self.requests_by_view_transport_method = self.register_metric(
            Counter,
            "django_http_requests_total_by_view_transport_method",
            "Count of requests by view, transport, method.",
            ["view", "transport", "method"],
            namespace=NAMESPACE,
        )
        self.requests_body_bytes = self.register_metric(
            Histogram,
            "django_http_requests_body_total_bytes",
            "Histogram of requests by body size.",
            buckets=PowersOf(2, 30),
            namespace=NAMESPACE,
        )
        # Set in process_template_response
        self.responses_by_templatename = self.register_metric(
            Counter,
            "django_http_responses_total_by_templatename",
            "Count of responses by template name.",
            ["templatename"],
            namespace=NAMESPACE,
        )
        # Set in process_response
        self.responses_by_status = self.register_metric(
            Counter,
            "django_http_responses_total_by_status",
            "Count of responses by status.",
            ["status"],
            namespace=NAMESPACE,
        )
        self.responses_by_status_view_method = self.register_metric(
            Counter,
            "django_http_responses_total_by_status_view_method",
            "Count of responses by status, view, method.",
            ["status", "view", "method"],
            namespace=NAMESPACE,
        )
        self.responses_body_bytes = self.register_metric(
            Histogram,
            "django_http_responses_body_total_bytes",
            "Histogram of responses by body size.",
            buckets=PowersOf(2, 30),
            namespace=NAMESPACE,
        )
        self.responses_by_charset = self.register_metric(
            Counter,
            "django_http_responses_total_by_charset",
            "Count of responses by charset.",
            ["charset"],
            namespace=NAMESPACE,
        )
        self.responses_streaming = self.register_metric(
            Counter,
            "django_http_responses_streaming_total",
            "Count of streaming responses.",
            namespace=NAMESPACE,
        )
        # Set in process_exception
        self.exceptions_by_type = self.register_metric(
            Counter,
            "django_http_exceptions_total_by_type",
            "Count of exceptions by object type.",
            ["type"],
            namespace=NAMESPACE,
        )
        self.exceptions_by_view = self.register_metric(
            Counter,
            "django_http_exceptions_total_by_view",
            "Count of exceptions by view.",
            ["view"],
            namespace=NAMESPACE,
        )


class PrometheusBeforeMiddleware(MiddlewareMixin):
    """Monitoring middleware that should run before other middlewares."""

    metrics_cls = Metrics

    def __init__(self, get_response=None):
        super().__init__(get_response)
        self.metrics = self.metrics_cls.get_instance()

    def process_request(self, request):
        self.metrics.requests_total.inc()
        request.prometheus_before_middleware_event = Time()

    def process_response(self, request, response):
        self.metrics.responses_total.inc()
        if hasattr(request, "prometheus_before_middleware_event"):
            self.metrics.requests_latency_before.observe(
                TimeSince(request.prometheus_before_middleware_event)
            )
        else:
            self.metrics.requests_unknown_latency_before.inc()
        return response


class PrometheusAfterMiddleware(MiddlewareMixin):
    """Monitoring middleware that should run after other middlewares."""

    metrics_cls = Metrics

    def __init__(self, get_response=None):
        super().__init__(get_response)
        self.metrics = self.metrics_cls.get_instance()

    def _transport(self, request):
        return "https" if request.is_secure() else "http"

    def _method(self, request):
        m = request.method
        if m not in (
            "GET",
            "HEAD",
            "POST",
            "PUT",
            "DELETE",
            "TRACE",
            "OPTIONS",
            "CONNECT",
            "PATCH",
        ):
            return "<invalid method>"
        return m

    def label_metric(self, metric, request, response=None, **labels):
        return metric.labels(**labels) if labels else metric

    def process_request(self, request):
        transport = self._transport(request)
        method = self._method(request)
        self.label_metric(self.metrics.requests_by_method, request, method=method).inc()
        self.label_metric(
            self.metrics.requests_by_transport, request, transport=transport
        ).inc()

        # Mimic the behaviour of the deprecated "Request.is_ajax()" method.
        if request.META.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest":
            self.label_metric(self.metrics.requests_ajax, request).inc()

        content_length = int(request.META.get("CONTENT_LENGTH") or 0)
        self.label_metric(self.metrics.requests_body_bytes, request).observe(
            content_length
        )
        request.prometheus_after_middleware_event = Time()

    def _get_view_name(self, request):
        view_name = "<unnamed view>"
        if hasattr(request, "resolver_match"):
            if request.resolver_match is not None:
                if request.resolver_match.view_name is not None:
                    view_name = request.resolver_match.view_name
        return view_name

    def process_view(self, request, view_func, *view_args, **view_kwargs):
        transport = self._transport(request)
        method = self._method(request)
        if hasattr(request, "resolver_match"):
            name = request.resolver_match.view_name or "<unnamed view>"
            self.label_metric(
                self.metrics.requests_by_view_transport_method,
                request,
                view=name,
                transport=transport,
                method=method,
            ).inc()

    def process_template_response(self, request, response):
        if hasattr(response, "template_name"):
            self.label_metric(
                self.metrics.responses_by_templatename,
                request,
                response=response,
                templatename=str(response.template_name),
            ).inc()
        return response

    def process_response(self, request, response):
        method = self._method(request)
        name = self._get_view_name(request)
        status = str(response.status_code)
        self.label_metric(
            self.metrics.responses_by_status, request, response, status=status
        ).inc()
        self.label_metric(
            self.metrics.responses_by_status_view_method,
            request,
            response,
            status=status,
            view=name,
            method=method,
        ).inc()
        if hasattr(response, "charset"):
            self.label_metric(
                self.metrics.responses_by_charset,
                request,
                response,
                charset=str(response.charset),
            ).inc()
        if hasattr(response, "streaming") and response.streaming:
            self.label_metric(self.metrics.responses_streaming, request, response).inc()
        if hasattr(response, "content"):
            self.label_metric(
                self.metrics.responses_body_bytes, request, response
            ).observe(len(response.content))
        if hasattr(request, "prometheus_after_middleware_event"):
            self.label_metric(
                self.metrics.requests_latency_by_view_method,
                request,
                response,
                view=self._get_view_name(request),
                method=request.method,
            ).observe(TimeSince(request.prometheus_after_middleware_event))
        else:
            self.label_metric(
                self.metrics.requests_unknown_latency, request, response
            ).inc()
        return response

    def process_exception(self, request, exception):
        self.label_metric(
            self.metrics.exceptions_by_type, request, type=type(exception).__name__
        ).inc()
        if hasattr(request, "resolver_match"):
            name = request.resolver_match.view_name or "<unnamed view>"
            self.label_metric(self.metrics.exceptions_by_view, request, view=name).inc()
        if hasattr(request, "prometheus_after_middleware_event"):
            self.label_metric(
                self.metrics.requests_latency_by_view_method,
                request,
                view=self._get_view_name(request),
                method=request.method,
            ).observe(TimeSince(request.prometheus_after_middleware_event))
        else:
            self.label_metric(self.metrics.requests_unknown_latency, request).inc()



from prometheus_client import Counter, Histogram

from django_prometheus.conf import NAMESPACE

connections_total = Counter(
    "django_db_new_connections_total",
    "Counter of created connections by database and by vendor.",
    ["alias", "vendor"],
    namespace=NAMESPACE,
)

connection_errors_total = Counter(
    "django_db_new_connection_errors_total",
    "Counter of connection failures by database and by vendor.",
    ["alias", "vendor"],
    namespace=NAMESPACE,
)

execute_total = Counter(
    "django_db_execute_total",
    (
        "Counter of executed statements by database and by vendor, including"
        " bulk executions."
    ),
    ["alias", "vendor"],
    namespace=NAMESPACE,
)


execute_many_total = Counter(
    "django_db_execute_many_total",
    ("Counter of executed statements in bulk operations by database and" " by vendor."),
    ["alias", "vendor"],
    namespace=NAMESPACE,
)


errors_total = Counter(
    "django_db_errors_total",
    ("Counter of execution errors by database, vendor and exception type."),
    ["alias", "vendor", "type"],
    namespace=NAMESPACE,
)

query_duration_seconds = Histogram(
    "django_db_query_duration_seconds",
    ("Histogram of query duration by database and vendor."),
    ["alias", "vendor"],
    namespace=NAMESPACE,
)



"""
Adapted from https://github.com/pygame/pygameweb/blob/master/pygameweb/builds/update_version_from_git.py

For updating the version from git.
__init__.py contains a __version__ field.
Update that.
If we are on master, we want to update the version as a pre-release.
git describe --tags
With these:
    __init__.py
        __version__= '0.0.2'
    git describe --tags
        0.0.1-22-g729a5ae
We want this:
    __init__.py
        __version__= '0.0.2.dev22.g729a5ae'
Get the branch/tag name with this.
    git symbolic-ref -q --short HEAD || git describe --tags --exact-match
"""

import os
import re
import subprocess


def migrate_source_attribute(attr, to_this, target_file, regex):
    """Updates __magic__ attributes in the source file"""
    change_this = re.compile(regex, re.S)
    new_file = []
    found = False

    with open(target_file, "r") as fp:
        lines = fp.readlines()

    for line in lines:
        if line.startswith(attr):
            found = True
            line = re.sub(change_this, to_this, line)
        new_file.append(line)

    if found:
        with open(target_file, "w") as fp:
            fp.writelines(new_file)


def migrate_version(target_file, new_version):
    """Updates __version__ in the source file"""
    regex = r"['\"](.*)['\"]"
    migrate_source_attribute(
        "__version__",
        "'{new_version}'".format(new_version=new_version),
        target_file,
        regex,
    )
    print("Updating to version: {}".format(new_version))


def is_master_branch():
    cmd = "git rev-parse --abbrev-ref HEAD"
    tag_branch = subprocess.check_output(cmd, shell=True)
    return tag_branch in [b"master\n"]


def git_tag_name():
    cmd = "git describe --tags"
    tag_branch = subprocess.check_output(cmd, shell=True)
    tag_branch = tag_branch.decode().strip()
    return tag_branch


def get_git_version_info():
    cmd = "git describe --tags"
    ver_str = subprocess.check_output(cmd, shell=True)
    ver, commits_since, githash = ver_str.decode().strip().split("-")
    if ver[0] == "v":
        ver = ver[1:]
    return ver, commits_since, githash


def prerelease_version():
    """ return what the prerelease version should be.
    https://packaging.python.org/tutorials/distributing-packages/#pre-release-versioning
    0.0.2.dev22
    """
    ver, commits_since, githash = get_git_version_info()
    initpy_ver = get_version()

    assert len(initpy_ver.split(".")) in [
        3,
        4,
    ], "django_prometheus/__init__.py version should be like 0.0.2.dev"
    assert (
        initpy_ver > ver
    ), "the django_prometheus/__init__.py version should be newer than the last tagged release."
    return "{initpy_ver}.{commits_since}".format(
        initpy_ver=initpy_ver, commits_since=commits_since
    )


def get_version():
    version_file = open("django_prometheus/__init__.py", "r").read()
    version_match = re.search(
        r'^__version__ = [\'"]([^\'"]*)[\'"]', version_file, re.MULTILINE
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def release_version_correct():
    """Makes sure the:
    - prerelease verion for master is correct.
    - release version is correct for tags.
    """
    if is_master_branch():
        # update for a pre release version.
        initpy = os.path.abspath("django_prometheus/__init__.py")

        new_version = prerelease_version()
        if ".dev" not in new_version:
            assert False, "Automation used only for pre-releases"
        print(
            "updating version in __init__.py to {new_version}".format(
                new_version=new_version
            )
        )
        assert (
            len(new_version.split(".")) >= 4
        ), "django_prometheus/__init__.py version should be like 0.0.2.dev"
        migrate_version(initpy, new_version)
    else:
        assert False, "No non-master deployments yet"
        # check that we are a tag with the same version as in __init__.py
        assert (
            get_version() == git_tag_name()
        ), "git tag/branch name not the same as django_prometheus/__init__.py __verion__"


if __name__ == "__main__":
    release_version_correct()



DJANGO_MIDDLEWARES = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.security.SecurityMiddleware",
]


def get_middleware(before, after):
    middleware = [before]
    middleware.extend(DJANGO_MIDDLEWARES)
    middleware.append(after)
    return middleware



#!/usr/bin/env python
import unittest
from operator import itemgetter

import prometheus_client

from django_prometheus.testutils import PrometheusTestCaseMixin


class SomeTestCase(PrometheusTestCaseMixin):
    """A class that pretends to be a unit test."""

    def __init__(self):
        self.passes = True
        super().__init__()

    def assertEqual(self, left, right, *args, **kwargs):
        self.passes = self.passes and (left == right)

    def assertFalse(self, expression, *args, **kwargs):
        self.passes = self.passes and (not expression)


class PrometheusTestCaseMixinTest(unittest.TestCase):
    def setUp(self):
        self.registry = prometheus_client.CollectorRegistry()
        self.some_gauge = prometheus_client.Gauge(
            "some_gauge", "Some gauge.", registry=self.registry
        )
        self.some_gauge.set(42)
        self.some_labelled_gauge = prometheus_client.Gauge(
            "some_labelled_gauge",
            "Some labelled gauge.",
            ["labelred", "labelblue"],
            registry=self.registry,
        )
        self.some_labelled_gauge.labels("pink", "indigo").set(1)
        self.some_labelled_gauge.labels("pink", "royal").set(2)
        self.some_labelled_gauge.labels("carmin", "indigo").set(3)
        self.some_labelled_gauge.labels("carmin", "royal").set(4)
        self.test_case = SomeTestCase()

    def testGetMetrics(self):
        """Tests getMetric."""
        assert 42 == self.test_case.getMetric("some_gauge", registry=self.registry)
        assert 1 == self.test_case.getMetric(
            "some_labelled_gauge",
            registry=self.registry,
            labelred="pink",
            labelblue="indigo",
        )

    def testGetMetricVector(self):
        """Tests getMetricVector."""
        vector = self.test_case.getMetricVector(
            "some_nonexistent_gauge", registry=self.registry
        )
        assert [] == vector
        vector = self.test_case.getMetricVector("some_gauge", registry=self.registry)
        assert [({}, 42)] == vector
        vector = self.test_case.getMetricVector(
            "some_labelled_gauge", registry=self.registry
        )
        assert (
            sorted(
                [
                    ({"labelred": "pink", "labelblue": "indigo"}, 1),
                    ({"labelred": "pink", "labelblue": "royal"}, 2),
                    ({"labelred": "carmin", "labelblue": "indigo"}, 3),
                    ({"labelred": "carmin", "labelblue": "royal"}, 4),
                ],
                key=itemgetter(1),
            )
            == sorted(vector, key=itemgetter(1))
        )

    def testAssertMetricEquals(self):
        """Tests assertMetricEquals."""
        # First we test that a scalar metric can be tested.
        self.test_case.assertMetricEquals(42, "some_gauge", registry=self.registry)
        assert self.test_case.passes is True
        self.test_case.assertMetricEquals(43, "some_gauge", registry=self.registry)
        assert self.test_case.passes is False
        self.test_case.passes = True

        # Here we test that assertMetricEquals fails on nonexistent gauges.
        self.test_case.assertMetricEquals(
            42, "some_nonexistent_gauge", registry=self.registry
        )
        assert not self.test_case.passes
        self.test_case.passes = True

        # Here we test that labelled metrics can be tested.
        self.test_case.assertMetricEquals(
            1,
            "some_labelled_gauge",
            registry=self.registry,
            labelred="pink",
            labelblue="indigo",
        )
        assert self.test_case.passes is True
        self.test_case.assertMetricEquals(
            1,
            "some_labelled_gauge",
            registry=self.registry,
            labelred="tomato",
            labelblue="sky",
        )
        assert self.test_case.passes is False
        self.test_case.passes = True

    def testRegistrySaving(self):
        """Tests saveRegistry and frozen registries operations."""
        frozen_registry = self.test_case.saveRegistry(registry=self.registry)
        # Test that we can manipulate a frozen scalar metric.
        assert 42 == self.test_case.getMetricFromFrozenRegistry(
            "some_gauge", frozen_registry
        )
        self.some_gauge.set(99)
        assert 42 == self.test_case.getMetricFromFrozenRegistry(
            "some_gauge", frozen_registry
        )
        self.test_case.assertMetricDiff(
            frozen_registry, 99 - 42, "some_gauge", registry=self.registry
        )
        assert self.test_case.passes is True
        self.test_case.assertMetricDiff(
            frozen_registry, 1, "some_gauge", registry=self.registry
        )
        assert self.test_case.passes is False
        self.test_case.passes = True

        # Now test the same thing with a labelled metric.
        assert 1 == self.test_case.getMetricFromFrozenRegistry(
            "some_labelled_gauge", frozen_registry, labelred="pink", labelblue="indigo"
        )
        self.some_labelled_gauge.labels("pink", "indigo").set(5)
        assert 1 == self.test_case.getMetricFromFrozenRegistry(
            "some_labelled_gauge", frozen_registry, labelred="pink", labelblue="indigo"
        )
        self.test_case.assertMetricDiff(
            frozen_registry,
            5 - 1,
            "some_labelled_gauge",
            registry=self.registry,
            labelred="pink",
            labelblue="indigo",
        )
        assert self.test_case.passes is True
        self.test_case.assertMetricDiff(
            frozen_registry,
            1,
            "some_labelled_gauge",
            registry=self.registry,
            labelred="pink",
            labelblue="indigo",
        )
        assert self.test_case.passes is False



from django.test import SimpleTestCase, override_settings
from prometheus_client import REGISTRY
from prometheus_client.metrics import MetricWrapperBase
from testapp.helpers import get_middleware
from testapp.test_middleware import M, T

from django_prometheus.middleware import (
    Metrics,
    PrometheusAfterMiddleware,
    PrometheusBeforeMiddleware,
)
from django_prometheus.testutils import PrometheusTestCaseMixin

EXTENDED_METRICS = [
    M("requests_latency_seconds_by_view_method"),
    M("responses_total_by_status_view_method"),
]


class CustomMetrics(Metrics):
    def register_metric(self, metric_cls, name, documentation, labelnames=(), **kwargs):
        if name in EXTENDED_METRICS:
            labelnames.extend(("view_type", "user_agent_type"))
        return super().register_metric(
            metric_cls, name, documentation, labelnames=labelnames, **kwargs
        )


class AppMetricsBeforeMiddleware(PrometheusBeforeMiddleware):
    metrics_cls = CustomMetrics


class AppMetricsAfterMiddleware(PrometheusAfterMiddleware):
    metrics_cls = CustomMetrics

    def label_metric(self, metric, request, response=None, **labels):
        new_labels = labels
        if metric._name in EXTENDED_METRICS:
            new_labels = {"view_type": "foo", "user_agent_type": "browser"}
            new_labels.update(labels)
        return super().label_metric(metric, request, response=response, **new_labels)


@override_settings(
    MIDDLEWARE=get_middleware(
        "testapp.test_middleware_custom_labels.AppMetricsBeforeMiddleware",
        "testapp.test_middleware_custom_labels.AppMetricsAfterMiddleware",
    )
)
class TestMiddlewareMetricsWithCustomLabels(PrometheusTestCaseMixin, SimpleTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Allow CustomMetrics to be used
        for metric in Metrics._instance.__dict__.values():
            if isinstance(metric, MetricWrapperBase):
                REGISTRY.unregister(metric)
        Metrics._instance = None

    def test_request_counters(self):
        registry = self.saveRegistry()
        self.client.get("/")
        self.client.get("/")
        self.client.get("/help")
        self.client.post("/", {"test": "data"})

        self.assertMetricDiff(registry, 4, M("requests_before_middlewares_total"))
        self.assertMetricDiff(registry, 4, M("responses_before_middlewares_total"))
        self.assertMetricDiff(registry, 3, T("requests_total_by_method"), method="GET")
        self.assertMetricDiff(registry, 1, T("requests_total_by_method"), method="POST")
        self.assertMetricDiff(
            registry, 4, T("requests_total_by_transport"), transport="http"
        )
        self.assertMetricDiff(
            registry,
            2,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.index",
            transport="http",
            method="GET",
        )
        self.assertMetricDiff(
            registry,
            1,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.help",
            transport="http",
            method="GET",
        )
        self.assertMetricDiff(
            registry,
            1,
            T("requests_total_by_view_transport_method"),
            view="testapp.views.index",
            transport="http",
            method="POST",
        )
        self.assertMetricDiff(
            registry,
            2.0,
            T("responses_total_by_status_view_method"),
            status="200",
            view="testapp.views.index",
            method="GET",
            view_type="foo",
            user_agent_type="browser",
        )
        self.assertMetricDiff(
            registry,
            1.0,
            T("responses_total_by_status_view_method"),
            status="200",
            view="testapp.views.help",
            method="GET",
            view_type="foo",
            user_agent_type="browser",
        )



from django.core.cache import caches
from django.test import TestCase
from redis import RedisError

from django_prometheus.testutils import PrometheusTestCaseMixin


class TestCachesMetrics(PrometheusTestCaseMixin, TestCase):
    """Test django_prometheus.caches metrics."""

    def testCounters(self):
        supported_caches = ["memcached", "filebased", "locmem", "redis"]

        # Note: those tests require a memcached server running
        for supported_cache in supported_caches:
            tested_cache = caches[supported_cache]

            total_before = (
                self.getMetric("django_cache_get_total", backend=supported_cache) or 0
            )
            hit_before = (
                self.getMetric("django_cache_get_hits_total", backend=supported_cache)
                or 0
            )
            miss_before = (
                self.getMetric("django_cache_get_misses_total", backend=supported_cache)
                or 0
            )

            tested_cache.set("foo1", "bar")
            tested_cache.get("foo1")
            tested_cache.get("foo1")
            tested_cache.get("foofoo")

            result = tested_cache.get("foofoo", default="default")

            assert result == "default"

            self.assertMetricEquals(
                total_before + 4, "django_cache_get_total", backend=supported_cache
            )
            self.assertMetricEquals(
                hit_before + 2, "django_cache_get_hits_total", backend=supported_cache
            )
            self.assertMetricEquals(
                miss_before + 2,
                "django_cache_get_misses_total",
                backend=supported_cache,
            )

    def test_redis_cache_fail(self):

        # Note: test use fake service config (like if server was stopped)
        supported_cache = "redis"

        total_before = (
            self.getMetric("django_cache_get_total", backend=supported_cache) or 0
        )
        fail_before = (
            self.getMetric("django_cache_get_fail_total", backend=supported_cache) or 0
        )
        hit_before = (
            self.getMetric("django_cache_get_hits_total", backend=supported_cache) or 0
        )
        miss_before = (
            self.getMetric("django_cache_get_misses_total", backend=supported_cache)
            or 0
        )

        tested_cache = caches["stopped_redis_ignore_exception"]
        tested_cache.get("foo1")

        self.assertMetricEquals(
            hit_before, "django_cache_get_hits_total", backend=supported_cache
        )
        self.assertMetricEquals(
            miss_before, "django_cache_get_misses_total", backend=supported_cache
        )
        self.assertMetricEquals(
            total_before + 1, "django_cache_get_total", backend=supported_cache
        )
        self.assertMetricEquals(
            fail_before + 1, "django_cache_get_fail_total", backend=supported_cache
        )

        tested_cache = caches["stopped_redis"]
        with self.assertRaises(RedisError):
            tested_cache.get("foo1")

        self.assertMetricEquals(
            hit_before, "django_cache_get_hits_total", backend=supported_cache
        )
        self.assertMetricEquals(
            miss_before, "django_cache_get_misses_total", backend=supported_cache
        )
        self.assertMetricEquals(
            total_before + 2, "django_cache_get_total", backend=supported_cache
        )
        self.assertMetricEquals(
            fail_before + 2, "django_cache_get_fail_total", backend=supported_cache
        )

    def test_cache_version_support(self):
        supported_caches = ["memcached", "filebased", "locmem", "redis"]

        # Note: those tests require a memcached server running
        for supported_cache in supported_caches:
            tested_cache = caches[supported_cache]

            tested_cache.set("foo1", "bar v.1", version=1)
            tested_cache.set("foo1", "bar v.2", version=2)

            assert "bar v.1" == tested_cache.get("foo1", version=1)
            assert "bar v.2" == tested_cache.get("foo1", version=2)
